/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

/**
 * Interface for java classes which act as plug-in game modifiers.
 */
public abstract class Plugin
{
   /**
    * The canonical name for a particular instance of the Plugin (as defined
    * in the config file).
    */
   private String canonicalName;
   
   /**
    * A reference back to the interface with the base system
    */
   protected PluginConnector vanilla;
   
   /**
    * Returns a canonical textual string representing the Plugin
    * @return The canonical textual string representing the Plugin.
    */
   protected String getPluginName() {return canonicalName;}

   /**
    * Sets the canonical string representing the Plugin.
	 * If you want to read config files you need a name, so this
	 * method is often overridden and used as a constructor [as
	 * we also load by reflection] if you don't forget to make a 
	 * super(s); call first thing in your function.
    * @param s The string representing the Plugin
    */
   protected void setPluginName(String s) {canonicalName = s;}
   
   /**
    * Sets the interface through which the Plugin communicates with the base
    * system.
    * @param p The PluginConnector object which deals with this Plugin
    */
   protected void setPluginConnector(PluginConnector p) {vanilla = p;}
   
   /**
    * Called when the game is first created, to allow the plugin to do any
    * initialisation necessary.
	 * Not currently used.
    */
   protected void gameCreated() {}
   
   /**
    * Called whenever a state change is added to a canonical event.
	 * Not currently used.
    * @param sc The state change which has been added.
    * @param qs If questions have been asked, this will contain the replies.
    * @see Plugin.getQuestions
    */
   protected void stateChangeAdded(StateChange sc, Vector qs)  throws AssassinSQLException{}
   
   /**
    * Called whenever an existing state change is about to be removed.
	 * Not currently used.
    * @param sc The state change which is about to be deleted.
    * @param qs If questions have been asked, this will contain the replies
    * @see Plugin.getQuestions
    */
   protected void stateChangeRemoved(StateChange sc, Vector qs)  throws AssassinSQLException{}

   /**
    * Called whenever the event to which an existing state change is
    * attached has its time changed.
	 * Not currently used.
    * @param sc The state change affected.
    * @param oldtime The previous timing of the state change's associated
    *    event.
    * @param newtime The new timing of the event. Note that this can also
    *    be retrieved with {@link StateChange#getEvent} and
    *    {@link Event#getTime}, but is provided here for convenience.
    */
   protected void stateChangeTimeChanged(StateChange sc, int oldtime, int newtime)  throws AssassinSQLException{}
   
   /**
    * Called whenever the plugin is required to update or sanity-check the
    * current gamestate.
	 * If someone runs the update-state command from the CLI, the 
	 * current state is built, and then each plugin in turn is
	 * asked to sanity check thsi and add any additional statechanges
	 * to an invisible event taking place at the time the command is 
	 * called.
	 * You can pass back a vector of quesitons. Once these have been 
	 * answered this function is called again and so on until it asks 
	 * no more questions.
	 * @param State a map from AssassinViewer =&gt; AssassinStateViewer
	 * @param qs a vector of answered questions
	 * @param e the event to add any statechanges to
	 * @return A Vector of questions you want answered.
    */
   protected Vector updateState(Map state, Vector qs, EventViewer e) throws AssassinSQLException, ParseException { return null; }
   
   /**
    * Called by the filtering mechanism when the plugin needs to determine
    * whether an assassin has a given property such as &quot;police&quot;. If
    * the plugin does not recognise the named property, it should return
    * <code>false</code>.
    * @param a A viewer for the assassin being tested.
    * @param prop The name of the property being asked about.
    * @return True if the assassin has the property, false if not.
    */
   public boolean assassinHasProperty(AssassinStateViewer a, String prop, String value)  throws AssassinSQLException{return false;}

   /**
    * Called during the updating of the gamestate to ask whether a given player
    * requires email to be sent to them during the current round of emails.
    * @param p A viewer for the player being asked about.
    * @return True if the plugin requires email to be sent to the player in
    *    question, false if not.
    */
   protected boolean playerNeedsEmail(AssassinStateViewer asv)  throws AssassinSQLException{return false;}

   /**
    * Called when the system has decided that email needs to be sent to a given
    * player. The plugin may return a string to be included in the email, or
    * null if it does not wish to include anything. This will be called 
	 * regardless of your plugin's answer to playerNeedsEmail if one is
	 * being sent anyway.
    * @param p A viewer for the player being emailed.
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
    * @return A string to be included in the email sent to the player, or null.
    *    The string will appear in its own paragraph when inserted into the
    *    email and so should not begin or end with a newline.
    */
   protected String generateEmail(AssassinStateViewer asv, Map state)  throws AssassinSQLException, ParseException {return null;}

   /**
    * Called when an event is being added to get any queries the plugin
    * has for the user. The plugin should check all the event details and if it
    * needs more information it should ask relevent questions. The 
    * returnQuestions function is passed the same Vector with the answers 
    * filled out.
    * @param e The Event which is being added
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
    * @return A Vector of Questions
    * @see returnQuestions
    */
   protected Vector getQuestions(EventViewer e, Map state) throws AssassinSQLException {return new Vector();}
   
   /**
    * Returns the Questions with the answers filled out
    *
    * @param e The Event being added
    * @param qs The Questions which were asked
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
	 * @see getQuestions
    */
   protected void returnQuestions(EventViewer e, Map state, Vector qs) throws AssassinSQLException {}

   /**
    * Called when an assassin is being added to get any queries the plugin
    * has for the user. The plugin should check all the assassin details and if it
    * needs more information it should ask relevent questions. The 
    * returnQuestions function is passed the same Vector with the answers 
    * filled out.
    * @param a The Assassin which is being added
    * @return A Vector of Questions
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
    * @see returnQuestions
    */
   protected Vector getQuestions(AssassinViewer a, Map state) throws AssassinSQLException {return new Vector();}
   
   /**
    * Returns the Questions with the answers filled out
    *
    * @param a The Assassin being added
    * @param e An Event to add StateChanges to
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
    * @param qs The Questions which were asked
	 * @see getQuestions
    */
   protected void returnQuestions(AssassinViewer a, EventViewer e, Map state, Vector qs) throws AssassinSQLException {}

   /**
    * Returns a Map of new commands to add to the CLI
    *
    * @return A Map of String =&gt; Command
    */
   public Map getCommands() { return new HashMap(); }

   /**
    * Returns a Map of new commands to add to the CGI
    *
    * @return A Map of String =&gt; CGICommand
    */
   public Map getCGICommands() { return new HashMap(); }

}


